@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *)
+*)
 
 (** Writing messages in the logs *)
 
@@ -28,47 +28,21 @@ val errlog : ?section:Lwt_log.section -> string -> unit
 (** Write a message in warnings.log *)
 val warning : ?section:Lwt_log.section -> string -> unit
 
-(** Write a message only in debugging mode (-V option) - Non cooperative *)
-val debug : (unit -> string) -> unit
-
-(** Write a message only in debugging mode (-V option) - Non cooperative *)
-val debug2 : string -> unit
-
-(** Same as [debug] without new line at the end - Non cooperative *)
-val debug_noel : (unit -> string) -> unit
-
-(** Same as [debug] without new line at the end - Non cooperative *)
-val debug_noel2 : string -> unit
-
 (** Write a message in the console (if not called in silent mode) *)
 val console : (unit -> string) -> unit
 
-(** Write a message in the console (if not called in silent mode) *)
-val console2 : string -> unit
-
 (** Use that function for all impossible cases in exception handlers
-   ([try ... with ... | e -> unexpected_exception ...] or [Lwt.catch ...]).
-   A message will be written in [warnings.log].
-   Put something in the string to help locating the problem (usually the name
-   of the function where is has been called).
- *)
+    ([try ... with ... | e -> unexpected_exception ...] or [Lwt.catch ...]).
+    A message will be written in [warnings.log].
+    Put something in the string to help locating the problem (usually the name
+    of the function where is has been called).
+*)
 val unexpected_exception : exn -> string -> unit
 
 (** Path to the error log file *)
 val error_log_path : unit -> string
 
-
-(** Register sections (so as to be able to dynamically change their level)
-    Level change is obtain by piping `logs: <section_name> <level_name>` to the
-    Ocsigen command-pipe. Parameter <section_name> must be the name of a
-    registered section (or else nothing happens), and <level_name> must be one
-    of `Debug`, `Info`, `Notice`, `Warning`, `Error`, `Fatal` (or else the
-    default level is restored)
- *)
-val register_section: Lwt_log.section -> unit
-
 (**/**)
 
 val open_files : ?user:string option -> ?group:string option -> unit -> unit Lwt.t
-val bip : int -> unit
 val command_f : exn -> string -> string list -> unit Lwt.t
