@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *)
+*)
 
 (* FIX: the log file is never reopened *)
 
@@ -25,9 +25,10 @@
 open Ocsidbmtypes
 open Lwt
 
+let section = Lwt_log.Section.make "ocsipersist:dbm"
 (** Data are divided into stores.
-   Create one store for your project, where you will save all your data.
- *)
+    Create one store for your project, where you will save all your data.
+*)
 type store = string
 
 exception Ocsipersist_error
@@ -42,24 +43,24 @@ open Simplexmlparser
 let rec parse_global_config (store, ocsidbm, delayloading as d) = function
   | [] -> d
   | Element ("delayloading", [("val", ("true" | "1"))], []) :: ll ->
-       parse_global_config (store, ocsidbm, true) ll
+    parse_global_config (store, ocsidbm, true) ll
 
   | Element ("store", [("dir", s)], []) :: ll ->
-      if store = None then
-        parse_global_config ((Some s), ocsidbm, delayloading) ll
-      else
-        Ocsigen_extensions.badconfig "Ocsipersist: Duplicate <store> tag"
+    if store = None then
+      parse_global_config ((Some s), ocsidbm, delayloading) ll
+    else
+      Ocsigen_extensions.badconfig "Ocsipersist: Duplicate <store> tag"
 
   | Element ("ocsidbm", [("name", s)], []) :: ll ->
-      if ocsidbm = None then
-        parse_global_config (store, (Some s), delayloading) ll
-      else
-        Ocsigen_extensions.badconfig "Ocsipersist: Duplicate <ocsidbm> tag"
+    if ocsidbm = None then
+      parse_global_config (store, (Some s), delayloading) ll
+    else
+      Ocsigen_extensions.badconfig "Ocsipersist: Duplicate <ocsidbm> tag"
 
   | (Element (s,_,_))::ll -> Ocsigen_extensions.badconfig "Bad tag %s" s
 
   | _ -> Ocsigen_extensions.badconfig
-      "Unexpected content inside Ocsipersist config"
+           "Unexpected content inside Ocsipersist config"
 
 let (directory, ocsidbm) =
   (ref ((Ocsigen_config.get_datadir ())^"/ocsipersist"),
@@ -74,12 +75,12 @@ external sys_exit : int -> 'a = "caml_sys_exit"
 let rec try_connect sname =
   catch
     (fun () ->
-      let socket = Lwt_unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 in
-      Lwt_unix.connect socket (Unix.ADDR_UNIX sname) >>= fun () ->
-      return socket)
+       let socket = Lwt_unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 in
+       Lwt_unix.connect socket (Unix.ADDR_UNIX sname) >>= fun () ->
+       return socket)
     (fun _ ->
-      Ocsigen_messages.warning ("Launching a new Ocsidbm process: "^(!ocsidbm)^
-                        " on directory "^(!directory)^".");
+       Lwt_log.ign_warning_f ~section
+         "Launching a new Ocsidbm process: %s on directory %s." !ocsidbm !directory;
       let param = [|!ocsidbm; !directory|] in
       let child () =
         let log =
@@ -93,10 +94,10 @@ let rec try_connect sname =
         Unix.close Unix.stdin;
         Unix.execv !ocsidbm param
       in
-      let pid = Unix.fork () in
+      let pid = Lwt_unix.fork () in
       if pid = 0
       then begin (* double fork *)
-        if Unix.fork () = 0
+        if Lwt_unix.fork () = 0
         then begin
           child ()
         end
@@ -116,15 +117,16 @@ let rec get_indescr i =
      (fun e ->
        if i = 0
        then begin
-         Ocsigen_messages.errlog ("Cannot connect to Ocsidbm. Will continue \
-                            without persistent session support. \
-                            Error message is: "^
-                            (match e with
-                            | Unix.Unix_error (a,b,c) ->
-                                (Unix.error_message a)^" in "^b^"("^c^")"
-                            | _ -> Printexc.to_string e)^
-                            ". Have a look at the logs to see if there is an \
-                            error message from the Ocsidbm process.");
+         Lwt_log.ign_error_f ~section
+           "Cannot connect to Ocsidbm. Will continue \
+            without persistent session support. \
+            Error message is: %s .\
+            Have a look at the logs to see if there is an \
+            error message from the Ocsidbm process."
+           (match e with
+            | Unix.Unix_error (a,b,c) ->
+              Printf.sprintf "%a in %s(%s)" (fun () -> Unix.error_message) a b c
+            | _ -> Printexc.to_string e);
          fail e
        end
        else (Lwt_unix.sleep 2.1) >>= (fun () -> get_indescr (i-1))))
@@ -137,26 +139,25 @@ let init_fun config =
     parse_global_config (None, None, false) config
   in
   (match store with
-     | None -> ()
-     | Some d -> directory := d);
+   | None -> ()
+   | Some d -> directory := d);
   (match ocsidbmconf with
-     | None -> ()
-     | Some d -> ocsidbm := d);
+   | None -> ()
+   | Some d -> ocsidbm := d);
 
-  Ocsigen_messages.warning
-    (if delay_loading then
-       "Asynchronuous initialization of Ocsipersist-dbm (may fail later)"
+  (if delay_loading then
+       Lwt_log.ign_warning ~section "Asynchronuous initialization (may fail later)"
      else
-       "Initializing Ocsipersist-dbm...");
+       Lwt_log.ign_warning ~section "Initializing ...");
   let indescr = get_indescr 2 in
   if delay_loading then (
     inch  := (indescr >>= fun r -> return (Lwt_chan.in_channel_of_descr r));
     outch := (indescr >>= fun r -> return (Lwt_chan.out_channel_of_descr r));
   ) else (
-    let r = Lwt_unix.run indescr in
+    let r = Lwt_main.run indescr in
     inch  := return (Lwt_chan.in_channel_of_descr r);
     outch := return (Lwt_chan.out_channel_of_descr r);
-    Ocsigen_messages.warning "...Initialization of Ocsipersist-dbm complete";
+    Lwt_log.ign_warning ~section "...Initialization complete";
   )
 
 
@@ -167,13 +168,13 @@ let send =
       (fun () -> !previous)
       (fun _ -> return Ok) >>=
     (fun _ ->
-      !inch >>= fun inch ->
-      !outch >>= fun outch ->
-      previous :=
-        (Lwt_chan.output_value outch v >>= fun () ->
-         Lwt_chan.flush outch >>= fun () ->
-         Lwt_chan.input_value inch);
-      !previous)
+       !inch >>= fun inch ->
+       !outch >>= fun outch ->
+       previous :=
+         (Lwt_chan.output_value outch v >>= fun () ->
+          Lwt_chan.flush outch >>= fun () ->
+          Lwt_chan.input_value inch);
+       !previous)
 
 let db_get (store, name) =
   send (Get (store, name)) >>=
@@ -244,10 +245,10 @@ let make_persistent_lazy_lwt ~store ~name ~default =
      (fun () -> db_get pvname >>= (fun _ -> return ()))
      (function
        | Not_found ->
-           default () >>= fun def ->
-           db_replace pvname (Marshal.to_string def [])
+         default () >>= fun def ->
+         db_replace pvname (Marshal.to_string def [])
        | e -> fail e)) >>=
-   (fun () -> return pvname)
+  (fun () -> return pvname)
 
 let make_persistent_lazy ~store ~name ~default =
   let default () = Lwt.wrap default in
@@ -305,7 +306,7 @@ let fold_table f table beg =
     (function
       | None -> return beg
       | Some k -> find table k >>= fun r ->
-          f k r beg >>= (fun res -> aux db_nextkey res))
+        f k r beg >>= (fun res -> aux db_nextkey res))
   in
   aux db_firstkey beg
 
@@ -314,13 +315,13 @@ let fold_step = fold_table
 let iter_block a b = failwith "iter_block not implemented for DBM. Please use Ocsipersist with sqlite"
 
 (* iterator: with a separate connexion:
-exception Exn1
-let iter_table f table =
-  let first = Marshal.to_string (Firstkey table) [] in
-  let firstl = String.length first in
-  let next = Marshal.to_string (Nextkey table) [] in
-  let nextl = String.length next in
-  (Lwt_unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 >>=
+   exception Exn1
+   let iter_table f table =
+   let first = Marshal.to_string (Firstkey table) [] in
+   let firstl = String.length first in
+   let next = Marshal.to_string (Nextkey table) [] in
+   let nextl = String.length next in
+   (Lwt_unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 >>=
    (fun socket ->
      Lwt_unix.connect
        (Lwt_unix.Plain socket)
