@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *)
+*)
 (*****************************************************************************)
 (*****************************************************************************)
 (* Ocsigen module to allow local (users) config files                        *)
@@ -30,6 +30,7 @@ open Ocsigen_extensions
 
 exception NoConfFile
 
+let section = Lwt_log.Section.make "ocsigen:ext:userconf"
 (*****************************************************************************)
 
 let err_500 =
@@ -39,16 +40,16 @@ let err_500 =
 (* Catch invalid userconf files and report an error *)
 let handle_parsing_error req = function
   | Ocsigen_extensions.Error_in_config_file s ->
-      Ocsigen_messages.errlog (Printf.sprintf
-          "Syntax error in userconf configuration file for url %s: %s"
-        req.request_info.ri_url_string s);
-      Lwt.return err_500
+    Lwt_log.ign_error_f ~section
+      "Syntax error in userconf configuration file for url %s: %s"
+      (Ocsigen_request_info.url_string req.request_info) s;
+    Lwt.return err_500
 
   | Ocsigen_extensions.Error_in_user_config_file s ->
-      Ocsigen_messages.errlog  (Printf.sprintf
-          "Unauthorized option in user configuration for url %s: %s"
-          req.request_info.ri_url_string s);
-      Lwt.return err_500
+    Lwt_log.ign_error_f ~section
+      "Unauthorized option in user configuration for url %s: %s"
+      (Ocsigen_request_info.url_string req.request_info) s;
+    Lwt.return err_500
 
   | e -> Lwt.fail e
 
@@ -69,23 +70,23 @@ let subresult new_req user_parse_site conf previous_err req req_state =
                and Ext_found_continue cases below) *)
             let rec aux ((answer, cts) as r) = match answer with
               | Ext_sub_result sr ->
-                  (* XXX Are these the good cookies ?? *)
-                  sr awake cookies_to_set req_state
-                  >>= aux
+                (* XXX Are these the good cookies ?? *)
+                sr awake cookies_to_set req_state
+                >>= aux
               | Ext_continue_with (newreq, cookies, err) ->
-                  Lwt.return
-                    ((Ext_continue_with
-                        ({req with request_config = newreq.request_config },
-                         cookies, err)), cts)
+                Lwt.return
+                  ((Ext_continue_with
+                      ({req with request_config = newreq.request_config },
+                       cookies, err)), cts)
               | Ext_found_continue_with r ->
-                  (* We keep config information outside userconf! *)
-                  Lwt.return
-                    (Ext_found_continue_with
-                       (fun () ->
-                          r () >>= fun (r, newreq) -> Lwt.return
+                (* We keep config information outside userconf! *)
+                Lwt.return
+                  (Ext_found_continue_with
+                     (fun () ->
+                        r () >>= fun (r, newreq) -> Lwt.return
                           (r,
                            { req with request_config = newreq.request_config })
-                       ), cts)
+                     ), cts)
               | _ -> Lwt.return r
             in aux (answer, cookies)
          )
@@ -99,23 +100,23 @@ let subresult new_req user_parse_site conf previous_err req req_state =
 let conf_to_xml conf =
   try Simplexmlparser.xmlparser_file conf
   with
-    | Sys_error _ -> raise NoConfFile
-    | Simplexmlparser.Xml_parser_error s ->
-        raise (Ocsigen_extensions.Error_in_config_file s)
+  | Sys_error _ -> raise NoConfFile
+  | Simplexmlparser.Xml_parser_error s ->
+    raise (Ocsigen_extensions.Error_in_config_file s)
 
 
 let gen hostpattern sitepath (regexp, conf, url, prefix, localpath) = function
   | Req_found _ ->
-      (* We do not allow setting filters through userconf files right now *)
-      Lwt.return Ext_do_nothing
+    (* We do not allow setting filters through userconf files right now *)
+    Lwt.return Ext_do_nothing
 
   | Req_not_found (previous_err, req) as req_state->
-      let path = req.request_info.ri_sub_path_string in
+      let path = (Ocsigen_request_info.sub_path_string req.request_info) in
       match Netstring_pcre.string_match regexp path 0 with
       | None -> Lwt.return (Ext_next previous_err)
       | Some _ ->
           try
-            Ocsigen_messages.debug2 "--Userconf: Using user configuration";
+            Lwt_log.ign_info ~section "Using user configuration";
             let conf0 = Ocsigen_extensions.replace_user_dir regexp conf path in
             let url = Netstring_pcre.global_replace regexp url path
             and prefix = Netstring_pcre.global_replace regexp prefix path
@@ -136,65 +137,80 @@ let gen hostpattern sitepath (regexp, conf, url, prefix, localpath) = function
             in
             let new_req =
               { req with request_info =
-                  { req.request_info with
-                      ri_sub_path = path; ri_sub_path_string = url}}
+                  (Ocsigen_request_info.update req.request_info
+                   ~sub_path:path
+                   ~sub_path_string:url ())}
             in
             Lwt.return
              (subresult new_req user_parse_site conf previous_err req req_state)
 
-          with
-          | Ocsigen_extensions.NoSuchUser
-          | NoConfFile
-          | Unix.Unix_error (Unix.EACCES,_,_)
-          | Unix.Unix_error (Unix.ENOENT, _, _) ->
-              Lwt.return (Ocsigen_extensions.Ext_next previous_err)
-          | e -> handle_parsing_error req e
+      with
+      | Ocsigen_extensions.NoSuchUser
+      | NoConfFile
+      | Unix.Unix_error (Unix.EACCES,_,_)
+      | Unix.Unix_error (Unix.ENOENT, _, _) ->
+        Lwt.return (Ocsigen_extensions.Ext_next previous_err)
+      | e -> handle_parsing_error req e
 
 
 
 (*****************************************************************************)
 (** Parsing of config file *)
-open Simplexmlparser
 
-let parse_config hostpattern _ path = fun _ _ ->
-  let rec parse_attrs_local l ((regexp, conf, url, prefix, path) as r) =
-    match l with
-    | [] -> r
-    | ("regexp", s)::l when regexp = None ->
-        (try parse_attrs_local l
-           (Some (Netstring_pcre.regexp ("^"^s^"$")), conf, url, prefix, path)
-         with Failure _ ->
-           badconfig "Bad regexp '%s' in <userconf regexp=\"...\" />" s)
-    | ("conf", s)::l when conf = None ->
-        parse_attrs_local l
-         (regexp, Some (Ocsigen_extensions.parse_user_dir s), url, prefix, path)
-    | ("url", s)::l when url = None ->
-        parse_attrs_local l (regexp, conf, Some s, prefix, path)
-    | ("prefix", s)::l when prefix = None ->
-        parse_attrs_local l (regexp, conf, url, Some s, path)
-    | ("localpath", s) :: l when path = None ->
-        parse_attrs_local l
-         (regexp, conf, url, prefix, Some (Ocsigen_extensions.parse_user_dir s))
-    | (a, _) :: _ ->
-        badconfig "Wrong or duplicate attribute %s for <userconf>" a
+let parse_config hostpattern _ path _ _ config_elem =
+  let regexp = ref None in
+  let conf = ref None in
+  let url = ref None in
+  let prefix = ref None in
+  let localpath = ref None in
+  Ocsigen_extensions.(
+    Configuration.process_element
+      ~in_tag:"host"
+      ~other_elements:(fun t _ _ -> raise (Bad_config_tag_for_extension t))
+      ~elements:[
+        Configuration.element
+          ~name:"userconf"
+          ~attributes:[
+            Configuration.attribute
+              ~name:"regexp"
+              ~obligatory:true
+              (fun s ->
+                 let s = Netstring_pcre.regexp ("^" ^ s ^ "$") in
+                 regexp := Some s);
+            Configuration.attribute
+              ~name:"conf"
+              ~obligatory:true
+              (fun s ->
+                 let s = Ocsigen_extensions.parse_user_dir s in
+                 conf := Some s);
+            Configuration.attribute
+              ~name:"url"
+              ~obligatory:true
+              (fun s -> url := Some s);
+            Configuration.attribute
+              ~name:"prefix"
+              ~obligatory:true
+              (fun s -> prefix := Some s);
+            Configuration.attribute
+              ~name:"localpath"
+              ~obligatory:true
+              (fun s ->
+                 let s = Ocsigen_extensions.parse_user_dir s in
+                 localpath := Some s)
+          ]
+          ()]
+      config_elem
+  );
+  let info =
+    match !regexp, !conf, !url, !prefix, !localpath  with
+    | (Some r, Some t, Some u, Some p, Some p') -> (r, t, u, p, p')
+    | _ -> badconfig "Missing attributes for <userconf>"
   in
-  function
-    | Element ("userconf", atts, []) ->
-        let info =
-          match parse_attrs_local atts (None, None, None, None, None)  with
-            | (Some r, Some t, Some u, Some p, Some p') -> (r, t, u, p, p')
-            | _ -> badconfig "Missing attributes for <userconf>"
-        in
-        gen hostpattern path info
-    | Element ("userconf", _, _ :: _) ->
-        badconfig "Incorrect (useless) data inside <userconf>"
-    | Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
-    | _ -> badconfig "Bad data in conf file"
-
+  gen hostpattern path info
 
 (*****************************************************************************)
 (** extension registration *)
 let () = register_extension
-  ~name:"userconf"
-  ~fun_site:parse_config
-  ()
+    ~name:"userconf"
+    ~fun_site:parse_config
+    ()
