@@ -1,7 +1,7 @@
 (* Ocsigen
  * http://www.ocsigen.org
  * Module ocsigen_parseconfig.ml
- * Copyright (C) 2005-2008 Vincent Balat, Nataliya Guts, Stphane Glondu
+ * Copyright (C) 2005-2008 Vincent Balat, Nataliya Guts, Stéphane Glondu
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,22 +16,25 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *)
+*)
 
 (******************************************************************)
 (** Config file parsing *)
 
 open Ocsigen_lib
+open Ocsigen_socket
 
 open Simplexmlparser
 open Ocsigen_config
 
+let section = Lwt_log.Section.make "ocsigen:config"
+
 let blah_of_string f tag s =
   try
     f (String.remove_spaces s 0 ((String.length s) -1))
   with Failure _ -> raise (Ocsigen_config.Config_file_error
                              ("While parsing <"^tag^"> - "^s^
-                                " is not a valid value."))
+                              " is not a valid value."))
 
 let int_of_string = blah_of_string int_of_string
 let float_of_string = blah_of_string float_of_string
@@ -40,18 +43,18 @@ let float_of_string = blah_of_string float_of_string
 let default_default_hostname =
   let hostname = Unix.gethostname () in
   try
-(*VVV Is it ok? Is it reliable? *)
+    (*VVV Is it ok? Is it reliable? *)
     (List.hd
        (Unix.getaddrinfo hostname "www"
           [Unix.AI_CANONNAME;
            Unix.AI_SOCKTYPE Unix.SOCK_STREAM])).Unix.ai_canonname
   with Failure _ ->
-    let warning =
-      "Cannot determine default host name. Will use \""^hostname^
-        "\" to create absolute links or redirections dynamically if you do not set <host defaulthostname=\"...\" ...> in config file."
-    in
-    Ocsigen_messages.warning warning;
-(*VVV Is it the right behaviour? *)
+    Lwt_log.ign_warning_f ~section
+      "Cannot determine default host name. Will use \"%s\" \
+       to create absolute links or redirections dynamically \
+       if you do not set <host defaulthostname=\"...\" ...> \
+       in config file." hostname;
+    (*VVV Is it the right behaviour? *)
     hostname
 (*****************************************************************************)
 
@@ -93,49 +96,49 @@ let parse_size =
            if (c1 = "T")
            then Int64.mul tebi (v l1)
            else
-             if (c1 = "G")
-             then Int64.mul gibi (v l1)
-             else
-               if (c1 = "M")
-               then Int64.mul mebi (v l1)
+           if (c1 = "G")
+           then Int64.mul gibi (v l1)
+           else
+           if (c1 = "M")
+           then Int64.mul mebi (v l1)
+           else
+           if (c1 = "k")
+           then Int64.mul kibi (v l1)
+           else
+             let l2 = l-2 in
+             if l2>0
+             then
+               let c2 = String.sub s l2 2 in
+               if (c2 = "To") || (c2 = "TB")
+               then Int64.mul tera (v l2)
                else
-                 if (c1 = "k")
-                 then Int64.mul kibi (v l1)
-                 else
-                   let l2 = l-2 in
-                   if l2>0
-                   then
-                     let c2 = String.sub s l2 2 in
-                     if (c2 = "To") || (c2 = "TB")
-                     then Int64.mul tera (v l2)
-                     else
-                       if (c2 = "Go") || (c2 = "GB")
-                       then Int64.mul giga (v l2)
-                       else
-                         if (c2 = "Mo") || (c2 = "MB")
-                         then Int64.mul mega (v l2)
-                         else
-                           if (c2 = "ko") || (c2 = "kB")
-                           then Int64.mul kilo (v l2)
-                           else
-                             let l3 = l-3 in
-                             if l3>0
-                             then
-                               let c3 = String.sub s l3 3 in
-                               if (c3 = "Tio") || (c3 = "TiB")
-                               then Int64.mul tebi (v l3)
-                               else
-                                 if (c3 = "Gio") || (c3 = "GiB")
-                                 then Int64.mul gibi (v l3)
-                                 else
-                                   if (c3 = "Mio") || (c3 = "MiB")
-                                   then Int64.mul mebi (v l3)
-                                   else
-                                     if (c3 = "kio") || (c3 = "kiB")
-                                     then Int64.mul kibi (v l3)
-                                     else o l
-                             else o l
+               if (c2 = "Go") || (c2 = "GB")
+               then Int64.mul giga (v l2)
+               else
+               if (c2 = "Mo") || (c2 = "MB")
+               then Int64.mul mega (v l2)
+               else
+               if (c2 = "ko") || (c2 = "kB")
+               then Int64.mul kilo (v l2)
+               else
+                 let l3 = l-3 in
+                 if l3>0
+                 then
+                   let c3 = String.sub s l3 3 in
+                   if (c3 = "Tio") || (c3 = "TiB")
+                   then Int64.mul tebi (v l3)
+                   else
+                   if (c3 = "Gio") || (c3 = "GiB")
+                   then Int64.mul gibi (v l3)
+                   else
+                   if (c3 = "Mio") || (c3 = "MiB")
+                   then Int64.mul mebi (v l3)
+                   else
+                   if (c3 = "kio") || (c3 = "kiB")
+                   then Int64.mul kibi (v l3)
                    else o l
+                 else o l
+             else o l
          else o l)
 
 
@@ -155,7 +158,7 @@ let parse_size_tag tag s =
 (* My xml parser is not really adapted to this.
    It is the parser for the syntax extension.
    But it works.
- *)
+*)
 
 
 let rec parse_string = function
@@ -178,23 +181,23 @@ let rec parser_config =
     | _ -> raise (Config_file_error "Don't know what to do with trailing data")
   in let rec parse_servers n = function
     | [] -> (match n with
-      | [] -> raise (Config_file_error ("<server> tag expected"))
-      | _ -> n)
+        | [] -> raise (Config_file_error ("<server> tag expected"))
+        | _ -> n)
     | (Element ("server", [], nouveau))::ll ->
         (match ll with
         | [] -> ()
         | _ ->
-            ignore (Ocsigen_messages.warning
-                      "At most one <server> tag possible in config file. \
-                      Ignoring trailing data."));
+          Lwt_log.ign_warning ~section
+            "At most one <server> tag possible in config file. \
+             Ignoring trailing data.");
         parse_servers (n@[nouveau]) [] (* ll *)
         (* Multiple server not supported any more *)
         (* nouveau at the end *)
     | _ -> raise (Config_file_error ("syntax error inside <ocsigen>"))
   in function
     | (Element ("ocsigen", [], l))::ll ->
-        verify_empty ll;
-        parse_servers [] l
+      verify_empty ll;
+      parse_servers [] l
     | _ -> raise (Config_file_error "<ocsigen> tag expected")
 
 
@@ -226,36 +229,36 @@ let parse_host_field =
        let r = match hostfilter with
          | None -> ["*", Netstring_pcre.regexp ".*$", None] (* default = "*:*" *)
          | Some s ->
-             let parse_one_host ss =
-               let host, port =
-                 try
-                   let dppos = String.index ss ':'
-                   and len = String.length ss in
-                   let host = String.sub ss 0 dppos
-                   and port =
-                     match String.sub ss (dppos+1) ((len - dppos) - 1) with
-                       | "*" -> None
-                       | p -> Some (int_of_string "host" p)
-                   in host, port
-                 with
-                   | Not_found -> ss, None
-                   | Failure _ ->
-                       raise (Config_file_error "bad port number")
-               in
-               let split_host = function
-                 | Netstring_str.Delim _ -> ".*"
-                 | Netstring_str.Text t -> Netstring_pcre.quote t
-               in
-               (host,
-                Netstring_pcre.regexp
-                  (String.concat ""
-                     ((List.map split_host
-                         (Netstring_str.full_split
-                            (Netstring_str.regexp "[*]+") host))@["$"])),
-                port)
+           let parse_one_host ss =
+             let host, port =
+               try
+                 let dppos = String.index ss ':'
+                 and len = String.length ss in
+                 let host = String.sub ss 0 dppos
+                 and port =
+                   match String.sub ss (dppos+1) ((len - dppos) - 1) with
+                   | "*" -> None
+                   | p -> Some (int_of_string "host" p)
+                 in host, port
+               with
+               | Not_found -> ss, None
+               | Failure _ ->
+                 raise (Config_file_error "bad port number")
              in
-             List.map parse_one_host
-               (Netstring_str.split (Netstring_str.regexp "[ \t]+") s)
+             let split_host = function
+               | Netstring_str.Delim _ -> ".*"
+               | Netstring_str.Text t -> Netstring_pcre.quote t
+             in
+             (host,
+              Netstring_pcre.regexp
+                (String.concat ""
+                   ((List.map split_host
+                       (Netstring_str.full_split
+                          (Netstring_str.regexp "[*]+") host))@["$"])),
+              port)
+           in
+           List.map parse_one_host
+             (Netstring_str.split (Netstring_str.regexp "[ \t]+") s)
        in
        Hashtbl.add h hostfilter r;
        (r : Ocsigen_extensions.virtual_hosts)
@@ -276,9 +279,9 @@ let get_defaulthostname ~defaulthostname ~defaulthttpport ~host =
             | _ :: q -> aux q
           in
           let host = aux host in
-          Ocsigen_messages.warning
-            ("While parsing config file, tag <host>: No defaulthostname, \
-              assuming it is \""^host^"\"");
+          Lwt_log.ign_warning_f ~section
+            "While parsing config file, tag <host>: No defaulthostname, \
+             assuming it is \"%s\"" host;
           if correct_hostname host then
              host
           else
@@ -288,7 +291,7 @@ let get_defaulthostname ~defaulthostname ~defaulthttpport ~host =
 
 (* Config file is parsed twice.
    This is the second parsing (site loading)
- *)
+*)
 let parse_server isreloading c =
   let rec parse_server_aux = function
       | [] -> []
@@ -558,13 +561,11 @@ let parse_server isreloading c =
                     let filename = dir^"/"^s in
                     let filecont =
                       try
-                        Ocsigen_messages.debug (fun () -> "Parsing configuration file "^
-                          filename);
+                        Lwt_log.ign_info_f ~section "Parsing configuration file %s" filename;
                         parse_ext filename
                       with e ->
-                        Ocsigen_messages.errlog
-                          ("Error while loading configuration file "^filename^
-                           ": "^(Printexc.to_string e)^" (ignored)");
+                        Lwt_log.ign_error_f ~section ~exn:e
+                          "Error while loading configuration file %s (ignored)" filename;
                         []
                     in
                     (match filecont with
@@ -577,10 +578,9 @@ let parse_server isreloading c =
                 files
             with
             | Sys_error _ as e ->
-                Ocsigen_messages.errlog
-                  ("Error while loading configuration file: "^
-                   ": "^(Printexc.to_string e)^" (ignored)");
-                []
+              Lwt_log.ign_error ~section ~exn:e
+                "Error while loading configuration file (ignored)";
+              []
           in
           one@(parse_server_aux ll)
       | (Element (tag, _, _))::_ ->
@@ -591,12 +591,6 @@ let parse_server isreloading c =
   in Ocsigen_extensions.set_hosts (parse_server_aux c)
 
 
-(* Types of socket declarable in configuration file *)
-type socket_type =
-  | IPv4 of Unix.inet_addr
-  | IPv6 of Unix.inet_addr
-  | All
-
 (* Parsing <port> tags *)
 let parse_port =
   let all_ipv6 = Netstring_pcre.regexp "^\\[::\\]:([0-9]+)$" in
@@ -607,16 +601,16 @@ let parse_port =
     let do_match r = Netstring_pcre.string_match r s 0 in
     let get x i = Netstring_pcre.matched_group x i s in
     match do_match all_ipv6 with
-      | Some r -> IPv6 (Unix.inet6_addr_any), int_of_string "port" (get r 1)
-      | None -> match do_match all_ipv4 with
+    | Some r -> IPv6 (Unix.inet6_addr_any), int_of_string "port" (get r 1)
+    | None -> match do_match all_ipv4 with
       | Some r -> IPv4 (Unix.inet_addr_any), int_of_string "port" (get r 1)
       | None -> match do_match single_ipv6 with
-      | Some r -> IPv6 (Unix.inet_addr_of_string (get r 1)),
-          int_of_string "port" (get r 2)
-      | None -> match do_match single_ipv4 with
-      | Some r -> IPv4 (Unix.inet_addr_of_string (get r 1)),
-          int_of_string "port" (get r 2)
-      | None -> All, int_of_string "port" s
+        | Some r -> IPv6 (Unix.inet_addr_of_string (get r 1)),
+                    int_of_string "port" (get r 2)
+        | None -> match do_match single_ipv4 with
+          | Some r -> IPv4 (Unix.inet_addr_of_string (get r 1)),
+                      int_of_string "port" (get r 2)
+          | None -> All, int_of_string "port" s
 
 let parse_facility = function
   | "auth" -> `Auth
@@ -641,115 +635,116 @@ let parse_facility = function
   | "security" -> `Security
   | "syslog" -> `Syslog
   | "uucp" -> `UUCP
-  | "user" -> `User;;
+  | "user" -> `User
+  | t -> raise (Config_file_error ("Unknown " ^ t ^ " facility in <syslog>"))
 
 (* First parsing of config file *)
 let extract_info c =
   let rec parse_ssl certificate privatekey = function
       [] -> Some (certificate,privatekey)
     | (Element ("certificate" as st, [], p))::l ->
-        (match certificate with
-          None ->
-            parse_ssl (Some (parse_string_tag st p)) privatekey l
-        | _ -> raise (Config_file_error
-                        "Two certificates inside <ssl>"))
+      (match certificate with
+         None ->
+         parse_ssl (Some (parse_string_tag st p)) privatekey l
+       | _ -> raise (Config_file_error
+                       "Two certificates inside <ssl>"))
     | (Element ("privatekey" as st, [], p))::l ->
-        (match privatekey with
-          None ->
-            parse_ssl certificate (Some (parse_string_tag st p)) l
-        | _ -> raise (Config_file_error
-                        "Two private keys inside <ssl>"))
+      (match privatekey with
+         None ->
+         parse_ssl certificate (Some (parse_string_tag st p)) l
+       | _ -> raise (Config_file_error
+                       "Two private keys inside <ssl>"))
     | (Element (tag,_,_))::l ->
-        raise (Config_file_error ("<"^tag^"> tag unexpected inside <ssl>"))
+      raise (Config_file_error ("<"^tag^"> tag unexpected inside <ssl>"))
     | _ -> raise (Config_file_error ("Unexpected content inside <ssl>"))
   in
   let rec aux user group ssl ports sslports minthreads maxthreads = function
       [] -> ((user, group), (ssl, ports,sslports), (minthreads, maxthreads))
     | (Element ("logdir" as st, [], p))::ll ->
-        set_logdir (parse_string_tag st p);
-        aux user group ssl ports sslports minthreads maxthreads ll
+      set_logdir (parse_string_tag st p);
+      aux user group ssl ports sslports minthreads maxthreads ll
     | (Element ("syslog" as st, [], p))::ll ->
-        let str = String.lowercase (parse_string_tag st p) in
-        set_syslog_facility (Some (parse_facility str));
-        aux user group ssl ports sslports minthreads maxthreads ll
+      let str = String.lowercase (parse_string_tag st p) in
+      set_syslog_facility (Some (parse_facility str));
+      aux user group ssl ports sslports minthreads maxthreads ll
     | (Element ("port" as st, atts, p))::ll ->
-        (match atts with
-          []
-        | [("protocol", "HTTP")] ->
-            let po = try
-              parse_port (parse_string_tag st p)
-            with Failure _ ->
-              raise (Config_file_error "Wrong value for <port> tag")
-            in aux user group ssl (po::ports) sslports minthreads maxthreads ll
-        | [("protocol", "HTTPS")] ->
-            let po = try
-              parse_port (parse_string_tag st p)
-            with Failure _ ->
-              raise (Config_file_error "Wrong value for <port> tag")
-            in
-            aux user group ssl ports (po::sslports) minthreads maxthreads ll
-        | _ -> raise (Config_file_error "Wrong attribute for <port>"))
+      (match atts with
+         []
+       | [("protocol", "HTTP")] ->
+         let po = try
+             parse_port (parse_string_tag st p)
+           with Failure _ ->
+             raise (Config_file_error "Wrong value for <port> tag")
+         in aux user group ssl (po::ports) sslports minthreads maxthreads ll
+       | [("protocol", "HTTPS")] ->
+         let po = try
+             parse_port (parse_string_tag st p)
+           with Failure _ ->
+             raise (Config_file_error "Wrong value for <port> tag")
+         in
+         aux user group ssl ports (po::sslports) minthreads maxthreads ll
+       | _ -> raise (Config_file_error "Wrong attribute for <port>"))
     | (Element ("minthreads" as st, [], p))::ll ->
-        aux user group ssl ports sslports
-          (Some (int_of_string st (parse_string_tag st p))) maxthreads ll
+      aux user group ssl ports sslports
+        (Some (int_of_string st (parse_string_tag st p))) maxthreads ll
     | (Element ("maxthreads" as st, [], p))::ll ->
-        aux user group ssl ports sslports minthreads
-          (Some (int_of_string st (parse_string_tag st p))) ll
+      aux user group ssl ports sslports minthreads
+        (Some (int_of_string st (parse_string_tag st p))) ll
     | (Element ("ssl", [], p))::ll ->
-        (match ssl with
-          None ->
-            aux user group (parse_ssl None None p) ports sslports
-              minthreads maxthreads ll
-        | _ ->
-            raise
-              (Config_file_error
-                 "Only one ssl certificate for each server supported for now"))
+      (match ssl with
+         None ->
+         aux user group (parse_ssl None None p) ports sslports
+           minthreads maxthreads ll
+       | _ ->
+         raise
+           (Config_file_error
+              "Only one ssl certificate for each server supported for now"))
     | (Element ("user" as st, [], p))::ll ->
-        (match user with
-          None ->
-            aux (Some (parse_string_tag st p)) group ssl ports sslports
-              minthreads maxthreads ll
-        | _ -> raise (Config_file_error
-                        "Only one <user> tag for each server allowed"))
+      (match user with
+         None ->
+         aux (Some (parse_string_tag st p)) group ssl ports sslports
+           minthreads maxthreads ll
+       | _ -> raise (Config_file_error
+                       "Only one <user> tag for each server allowed"))
     | (Element ("group" as st, [], p))::ll ->
-        (match group with
-          None ->
-            aux user (Some (parse_string_tag st p)) ssl ports sslports
-              minthreads maxthreads ll
-        | _ -> raise (Config_file_error
-                        "Only one <group> tag for each server allowed"))
+      (match group with
+         None ->
+         aux user (Some (parse_string_tag st p)) ssl ports sslports
+           minthreads maxthreads ll
+       | _ -> raise (Config_file_error
+                       "Only one <group> tag for each server allowed"))
     | (Element ("commandpipe" as st, [], p))::ll ->
-        set_command_pipe (parse_string_tag st p);
-        aux user group ssl ports sslports minthreads maxthreads ll
+      set_command_pipe (parse_string_tag st p);
+      aux user group ssl ports sslports minthreads maxthreads ll
     | (Element (tag, _, _))::ll ->
-        aux user group ssl ports sslports minthreads maxthreads ll
+      aux user group ssl ports sslports minthreads maxthreads ll
     | _ ->
-        raise (Config_file_error "Syntax error")
+      raise (Config_file_error "Syntax error")
   in
   let (user, group), si, (mint, maxt) = aux None None None [] [] None None c in
   let user = match user with
-    None -> None (* Some (get_default_user ()) *)
-  | Some s -> if s = "" then None else Some s
+      None -> None (* Some (get_default_user ()) *)
+    | Some s -> if s = "" then None else Some s
   in
   let group = match group with
-    None -> None (* Some (get_default_group ()) *)
-  | Some s -> if s = "" then None else Some s
+      None -> None (* Some (get_default_group ()) *)
+    | Some s -> if s = "" then None else Some s
   in
   let mint = match mint with
-  | Some t -> t
-  | None -> get_minthreads ()
+    | Some t -> t
+    | None -> get_minthreads ()
   in
   let maxt = match maxt with
-  | Some t -> t
-  | None -> get_maxthreads ()
+    | Some t -> t
+    | None -> get_maxthreads ()
   in
   ((user, group), si, (mint, maxt))
 
 let parse_config ?file () =
   let file =
     match file with
-      | None -> Ocsigen_config.get_config_file ()
-      | Some f -> f
+    | None -> Ocsigen_config.get_config_file ()
+    | Some f -> f
   in
   parser_config (Simplexmlparser.xmlparser_file file)
 
